@@ -28,6 +28,12 @@ class BitArray(object):
         if size > len(self.array):
             self.array = ([0] * (size - len(self.array))) + self.array
 
+    def copy(self):
+        new = type(self)(0, "")
+        new.size = self.size
+        new.array = list(self.array)
+        return new
+
     def dump(self):
         res = []
         for i in range(self.size // 8):
@@ -283,7 +289,7 @@ RegisterEax = lambda: FixedRegister('EAX')
 
 class RawBits(BitArray):
     def accept_arg(self, args, instr_state):
-        return (0, self)
+        return (0, self.copy())
 
 
 # Immediat value logic
@@ -404,8 +410,6 @@ class SegmentSelectorAbsoluteAddr(object):
 
 
 
-
-
 class ModRM(object):
     def __init__(self, sub_modrm, accept_reverse=True, has_direction_bit=True):
         self.accept_reverse = accept_reverse
@@ -438,7 +442,6 @@ class ModRM_REG__REG(object):
     def match(cls, arg1, arg2):
         return X86.is_reg(arg1) and X86.is_reg(arg2)
 
-
     def __init__(self, arg1, arg2, reversed, instr_state):
         self.mod = BitArray(2, "11")
         if X86.reg_size(arg1) != X86.reg_size(arg2):
@@ -601,6 +604,7 @@ class Instruction(object):
     encoding = []
 
     def __init__(self, *initial_args):
+        # print(self, initial_args)
         for type_encoding in self.encoding:
             args = list(initial_args)
             prefix = []
@@ -620,6 +624,7 @@ class Instruction(object):
         raise ValueError("Cannot encode <{0} {1}>:(".format(type(self).__name__, initial_args))
 
     def get_code(self):
+        # print(self.value)
         prefix_opcode = b"".join(chr(p.PREFIX_VALUE) for p in self.prefix)
         return prefix_opcode + bytes(self.value.dump())
 
@@ -712,6 +717,9 @@ class Jz(JmpType):
                 (RawBits.from_int(16, 0x0f84), JmpImm32(6))]
 
 
+Je = Jz
+
+
 class Jnz(JmpType):
     encoding = [(RawBits.from_int(8, 0x75), JmpImm8(2)),
                 (RawBits.from_int(16, 0x0f85), JmpImm32(6))]
@@ -788,6 +796,22 @@ class Movsd(Instruction):
     encoding = [(RawBits.from_int(8, 0xa5),)]
 
 
+class Pushfd(Instruction):
+    encoding = [(RawBits.from_int(8, 0x9c),)]
+
+
+class Pushad(Instruction):
+    encoding = [(RawBits.from_int(8, 0x60),)]
+
+
+class Popfd(Instruction):
+    encoding = [(RawBits.from_int(8, 0x9d),)]
+
+
+class Popad(Instruction):
+    encoding = [(RawBits.from_int(8, 0x61),)]
+
+
 class Lea(Instruction):
     encoding = [(RawBits.from_int(8, 0x8d), ModRM([ModRM_REG__MEM], accept_reverse=False, has_direction_bit=False))]
 
@@ -799,7 +823,7 @@ class Cmp(Instruction):
 
 
 class Test(Instruction):
-    encoding = [(RawBits.from_int(8, 0xf7), Slash(7), Imm32()),
+    encoding = [(RawBits.from_int(8, 0xf7), Slash(0), Imm32()),
                 (RawBits.from_int(8, 0x85), ModRM([ModRM_REG__REG, ModRM_REG__MEM], has_direction_bit=False))]
 
 
@@ -1104,6 +1128,13 @@ def assemble_instructions_generator(str):
         except:
             raise ValueError("Unknow mnemonic <{0}>".format(mnemo))
 
+        if issubclass(instr_object, Raw):
+            # Raw should received the raw buffer as it expect encoded hex
+            # The transformation may transform 'raw 9090' (nopnop) as 0n9090
+            # If other fake-instr need this : make a class attribute
+            yield instr_object(*args_raw)
+            continue
+
         args = []
         if args_raw:
             for arg in args_raw[0].split(","):
