@@ -285,6 +285,8 @@ MANDATORY_LABEL_ACE_MASK = gdef.FlagMapper(
     gdef.SYSTEM_MANDATORY_LABEL_NO_EXECUTE_UP,
 )
 
+
+
 class AceHeader(gdef.ACE_HEADER):
     """Improved ACE_HEADER"""
     def _to_ace_type(self, ace_type):
@@ -715,6 +717,12 @@ class SecurityDescriptor(gdef.PSECURITY_DESCRIPTOR):
 
     _close_function = winproxy.LocalFree
 
+    @classmethod
+    def create(cls):
+        buff = windows.utils.BUFFER(gdef.SECURITY_DESCRIPTOR, 1)()
+        winproxy.InitializeSecurityDescriptor(buff, gdef.SECURITY_DESCRIPTOR_REVISION)
+        return buff.cast(windows.security.SecurityDescriptor)
+
     @property
     def control(self):
         """The security descriptor control
@@ -737,8 +745,7 @@ class SecurityDescriptor(gdef.PSECURITY_DESCRIPTOR):
         winproxy.GetSecurityDescriptorControl(self, control, lpdwRevision)
         return lpdwRevision.value
 
-    @property
-    def owner(self):
+    def get_owner(self):
         """The owner of the security descriptor
 
         :type: :class:`~windows.generated_def.winstructs.PSID` or ``None``
@@ -749,6 +756,11 @@ class SecurityDescriptor(gdef.PSECURITY_DESCRIPTOR):
         # Return None of owner is NULL
         return owner or None
 
+    def set_owner(self, new_owner):
+        winproxy.SetSecurityDescriptorOwner(self, new_owner, bOwnerDefaulted=False)
+
+    owner = property(get_owner, set_owner)
+
     @property
     def group(self):
         """The group of the security descriptor
@@ -804,8 +816,8 @@ class SecurityDescriptor(gdef.PSECURITY_DESCRIPTOR):
             See `ConvertStringSecurityDescriptorToSecurityDescriptorA <https://docs.microsoft.com/en-us/windows/desktop/api/sddl/nf-sddl-convertstringsecuritydescriptortosecuritydescriptora>`_
         """
         self = cls()
-        winproxy.ConvertStringSecurityDescriptorToSecurityDescriptorA(
-            sddl.encode("ascii"),
+        winproxy.ConvertStringSecurityDescriptorToSecurityDescriptorW(
+            sddl,
             gdef.SDDL_REVISION_1,
             self,
             None)
@@ -820,7 +832,7 @@ class SecurityDescriptor(gdef.PSECURITY_DESCRIPTOR):
         if query_sacl:
             flags |= gdef.SACL_SECURITY_INFORMATION
 
-        winproxy.GetNamedSecurityInfoA(
+        winproxy.GetNamedSecurityInfoW(
             objname,
             objtype,
             flags,
@@ -930,8 +942,8 @@ class SecurityDescriptor(gdef.PSECURITY_DESCRIPTOR):
 
         :type: :class:`str`
         """
-        result_cstr = gdef.LPSTR()
-        winproxy.ConvertSecurityDescriptorToStringSecurityDescriptorA(
+        result_cstr = gdef.LPWSTR()
+        winproxy.ConvertSecurityDescriptorToStringSecurityDescriptorW(
             self,
             gdef.SDDL_REVISION_1,
             security_information,
@@ -939,15 +951,70 @@ class SecurityDescriptor(gdef.PSECURITY_DESCRIPTOR):
             None)
         result = result_cstr.value # Retrieve a python-str copy
         winproxy.LocalFree(result_cstr)
-        return result.decode()
+        return result
 
     __str__ = to_string
 
+    def isrelative(self):
+        """[WIP] api may change"""
+        return bool(self.control & gdef.SE_SELF_RELATIVE)
 
-    # TST
+    def make_absolute(self):
+        """[WIP] api may change"""
+        # Assert relative ?
+        abs_descriptor_size = gdef.DWORD(0)
+        dacl_size =  gdef.DWORD(0)
+        sacl_size =  gdef.DWORD(0)
+        owner_size =  gdef.DWORD(0)
+        prim_groupe_size =  gdef.DWORD(0)
+
+        try:
+            windows.winproxy.MakeAbsoluteSD(self, None, abs_descriptor_size,
+                    None,
+                    dacl_size,
+                    None,
+                    sacl_size,
+                    None,
+                    owner_size,
+                    None,
+                    prim_groupe_size)
+        except WindowsError as e:
+            if e.winerror != gdef.ERROR_INSUFFICIENT_BUFFER:
+                raise
+
+        abs_sd = windows.utils.BUFFER(gdef.BYTE, abs_descriptor_size.value)().cast(SecurityDescriptor)
+        # dacl = ctypes.create_string_buffer(dacl_size.value)
+        dacl = windows.utils.BUFFER(gdef.BYTE, dacl_size.value)()
+        sacl = windows.utils.BUFFER(gdef.BYTE, sacl_size.value)()
+        owner = ctypes.create_string_buffer(owner_size.value)
+        prim_groupe = ctypes.create_string_buffer(prim_groupe_size.value)
+
+        windows.winproxy.MakeAbsoluteSD(self, abs_sd, abs_descriptor_size,
+                dacl.cast(gdef.PACL),
+                dacl_size,
+                sacl.cast(gdef.PACL),
+                sacl_size,
+                owner,
+                owner_size,
+                prim_groupe,
+                prim_groupe_size)
+
+        abs_sd._save = [dacl, sacl, owner, prim_groupe]
+        return abs_sd
+
+    def make_relative(self):
+        """[WIP] api may change"""
+        srsd_size =  gdef.DWORD(0)
+        try:
+            windows.winproxy.MakeSelfRelativeSD(self, None, srsd_size)
+        except WindowsError as e:
+            if e.winerror != gdef.ERROR_INSUFFICIENT_BUFFER:
+                raise
+
+        srsd = windows.utils.BUFFER(gdef.BYTE, srsd_size.value)().cast(SecurityDescriptor)
+        windows.winproxy.MakeSelfRelativeSD(self, srsd, srsd_size)
+        return srsd
 
-    # def relative(self):
-        # return bool(self.control & gdef.SE_SELF_RELATIVE)
 
     # If we want auto-free we need to handle relf-relative SD
     # We need to keep-track of sub-object of the SD
@@ -958,6 +1025,11 @@ class SecurityDescriptor(gdef.PSECURITY_DESCRIPTOR):
             # print("FREE SELF")
             # self._close_function(self)
 
+    def __repr__(self):
+        if (self.isrelative()):
+            return "<{0} (self relative format) at {1:#x}>".format(type(self).__name__, id(self))
+        return "<{0} (absolute format) at {1:#x}>".format(type(self).__name__, id(self))
+
     def explain(self, type=None):
         if type is  None:
             type = getattr(self, "obj_type", None)
@@ -1043,15 +1115,37 @@ def explain_callback_ace(ace, sdtype):
     yield u"Mask:"
     mapper = windows.security.SPECIFIC_ACCESS_RIGTH_BY_TYPE[sdtype]
     yield u"  " + str(list(mapper[x] for x in ace.mask))
-    yield "Application data:"
-    yield "  " + repr(ace.application_data.replace("\x00", ""))
+    yield u"Application data:"
+    yield u"  " + repr(ace.application_data.replace(b"\x00", b""))
     try:
         condition = ace.condition
-        yield "Condition:"
-        yield "  " + condition
+        yield u"Condition:"
+        yield u"  " + condition
     except ValueError:
         pass
 
+def explain_resource_attribute(ace, sdtype):
+    yield u"Type:"
+    yield u"  " + str(ace.Header.AceType)
+    yield u"HeaderFlags:"
+    yield u"  {0:#x}".format(ace.Header.AceFlags)
+    yield u"SID:"
+    yield u"  " + explain_sid(ace.sid)
+    yield u"Mask:"
+    mapper = windows.security.SPECIFIC_ACCESS_RIGTH_BY_TYPE[sdtype]
+    yield u"  " + str(list(mapper[x] for x in ace.mask))
+
+    attribute = ace.attribute
+    yield u"Attribute Name:"
+    yield u"  " + attribute.name
+    yield u"Attribute Type:"
+    yield u"  {0:#x}".format(attribute.ValueType)
+    yield u"Attribute Flags:"
+    yield u"  {0:#x}".format(attribute.Flags)
+    yield u"Values:"
+    for value in attribute.values:
+        yield u"  " + repr(value)
+
 
 ACE_EXPLAINER = {
     gdef.ACCESS_ALLOWED_ACE_TYPE: explain_simple_ace,
@@ -1061,6 +1155,7 @@ ACE_EXPLAINER = {
     gdef.ACCESS_ALLOWED_OBJECT_ACE_TYPE: explain_object_related_ace,
     gdef.ACCESS_DENIED_OBJECT_ACE_TYPE: explain_object_related_ace,
     gdef.ACCESS_ALLOWED_CALLBACK_ACE_TYPE: explain_callback_ace,
+    gdef.SYSTEM_RESOURCE_ATTRIBUTE_ACE_TYPE: explain_resource_attribute,
 }
 
 
