@@ -66,18 +66,19 @@ class EvtHandle(gdef.EVT_HANDLE):
 # Class high-level API
 class EvtQuery(EvtHandle):
     """Represent an Event-log query"""
-    TIMEOUT = 0x1000
+    DEFAULT_TIMEOUT = 0x1000
 
-    def __init__(self, handle=0, channel=None):
+    def __init__(self, handle=0, channel=None, timeout=None):
         super(EvtQuery, self).__init__(handle)
         self.channel = channel
+        self.timeout = timeout if timeout is not None else self.DEFAULT_TIMEOUT
 
     def __next__(self):
         """Return the next :class:`EvtEvent` matching the query"""
         try:
             event = EvtEvent(channel=self.channel)
             ret = gdef.DWORD()
-            windows.winproxy.EvtNext(self, 1, event, self.TIMEOUT, 0, ret)
+            windows.winproxy.EvtNext(self, 1, event, self.timeout, 0, ret)
         except WindowsError as e:
             if e.winerror == gdef.ERROR_NO_MORE_ITEMS:
                 raise StopIteration
@@ -437,7 +438,7 @@ class EvtChannel(object):
         self.event_metadata_by_id = {}
         self.classic_event_metadata_by_id = {} # For classic only
 
-    def query(self, filter=None, ids=None):
+    def query(self, filter=None, ids=None, timeout=None):
         """Query the event with the ``ids`` or perform a query with the raw query ``filter``
 
         Both parameters are mutually exclusive.
@@ -456,6 +457,9 @@ class EvtChannel(object):
             List all event with a given EventID while searching for a specific field value (Sysmon for the test here)
                 ``Event/System[EventID=3] and Event/EventData/Data[@Name='DestinationIp'] and Event/EventData[Data='10.0.0.2']``
 
+            List all events with a given provider of Microsoft-Windows-TaskScheduler:
+                ``Event/System/Provider[@Name='Microsoft-Windows-TaskScheduler']``
+
         :rtype: :class:`EvtQuery`
         """
         if ids and filter:
@@ -466,7 +470,7 @@ class EvtChannel(object):
             ids_filter = " or ".join("EventID={0}".format(id) for id in ids)
             filter = "Event/System[{0}]".format(ids_filter)
         query_handle = winproxy.EvtQuery(None, self.name, filter, self.DEFAULT_QUERY_FLAGS)
-        return EvtQuery(query_handle, self)
+        return EvtQuery(query_handle, self, timeout=timeout)
 
     @property
     def events(self):
@@ -526,6 +530,9 @@ class EvtFile(EvtChannel):
 
 class ChannelConfig(EvtHandle):
     """The configuration of a event channel"""
+
+    # TODO: https://learn.microsoft.com/en-us/windows/win32/api/winevt/nf-winevt-evtsetchannelconfigproperty
+
     def __init__(self, handle, name=None):
         super(ChannelConfig, self).__init__(handle)
         self.name = name
@@ -541,7 +548,10 @@ class ChannelConfig(EvtHandle):
         return EvtPublisher(chaninfo(self, gdef.EvtChannelConfigOwningPublisher).value)
 
     def publishers(self):
-        "TEST"
+        """ The list of Publisher publishing on the channel. seems pertinent for Legacy channel like "System"
+
+        :type: [:class:`EvtPublisher`] - A list of :class:`EvtPublisher`
+        """
         return [EvtPublisher(pub) for pub in chaninfo(self, gdef.EvtChannelPublisherList).value]
 
     @property
